using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.Windows.Media.Media3D;

namespace UGDemo
{
	public class GardenViewPanel3D : Panel
	{
		private Viewport3D _viewport;
		private Model3DGroup _modelGroup;
		private Random _random = new Random();

		public static readonly DependencyProperty ElementWidthProperty = DependencyProperty.Register(
			"ElementWidth", typeof (double), typeof (GardenViewPanel3D));

		public static readonly DependencyProperty ElementHeightProperty = DependencyProperty.Register(
			"ElementHeight", typeof (double), typeof (GardenViewPanel3D));

		private static readonly DependencyProperty ItemIndexProperty = DependencyProperty.Register(
			"ItemIndex", typeof (int), typeof (GardenViewPanel3D));

		public double ElementWidth
		{
			get { return (double) GetValue(ElementWidthProperty); }
			set { SetValue(ElementWidthProperty, value); }
		}

		public double ElementHeight
		{
			get { return (double) GetValue(ElementHeightProperty); }
			set { SetValue(ElementHeightProperty, value); }
		}


		public GardenViewPanel3D()
		{
			CreateViewport();
		}

		private void CreateViewport()
		{
			_viewport = ResourceManager.Get<Viewport3D>("3DViewport");
			_modelGroup = LocateModelGroup();

		}

		private Model3DGroup LocateModelGroup()
		{
			Model3DGroup group = ((_viewport.Children[0] as ModelVisual3D).Content as Model3DGroup).Children[1] as Model3DGroup;
			return group;
		}

		#region Layout Overrides
		protected override Size MeasureOverride(Size availableSize)
		{
			_viewport.Measure(availableSize);

			foreach (UIElement child in Children)
			{
				child.Measure(new Size(ElementWidth, ElementHeight));
			}
			return availableSize;
		}

		protected override Size ArrangeOverride(Size finalSize)
		{
			_viewport.Arrange(new Rect(finalSize));

			foreach (UIElement child in Children)
			{
				child.Arrange(new Rect(new Size(ElementWidth, ElementHeight)));
			}
			return finalSize;
		}

		protected override Visual GetVisualChild(int index)
		{
			if (index == 0) return _viewport;
			else throw new Exception("Bad Index");
		}

		protected override int VisualChildrenCount
		{
			get
			{
				// HILITE: Can't return 1 always, need to check the parent's count first
				int count = base.VisualChildrenCount == 0 ? 0 : 1;
				return count;
			}
		}

		protected override void OnVisualChildrenChanged(DependencyObject visualAdded, DependencyObject visualRemoved)
		{
			base.OnVisualChildrenChanged(visualAdded, visualRemoved);

			if (visualAdded != null)
			{
				int index = InternalChildren.IndexOf(visualAdded as UIElement);
				GeometryModel3D model = CreateModel(visualAdded);
				model.SetValue(ItemIndexProperty, index);
				_modelGroup.Children.Add(model);
			}
			if (visualRemoved != null)
			{
				int index = (int) visualRemoved.GetValue(ItemIndexProperty);

				GeometryModel3D model = _modelGroup.Children[index] as GeometryModel3D;
				model.Material = null;

				_modelGroup.Children.Remove(model);
			}
		}

		private GeometryModel3D CreateModel(DependencyObject visualAdded)
		{
			int index = Math.Max(0, InternalChildren.Count - 1);
			GeometryModel3D model = ResourceManager.Get<GeometryModel3D>("PlaneModel").Clone();
			(model.Material as DiffuseMaterial).Brush = (new VisualBrush(visualAdded as Visual));
			double zPos = -1* index;
			double xPos = 0.5*index;

			TranslateTransform3D trans = ((model.Transform as Transform3DGroup).Children[1] as TranslateTransform3D);
			trans.OffsetX = xPos;
			trans.OffsetZ = zPos;
			return model;
		}

		#endregion
	}
}